from .answers import answers
from .questions import questions
from .shakes import shakes
import random
import time

# return list of sample questions, either to the specific count or all available questions
def get_questions(language, count = 0):
    if language not in questions.keys():
        # return error message that specific language is not available
        return []
    
    sample_questions = []
    # return array with all sample questions
    if count == 0 or count > len(questions[language]):
        for question in questions[language]:
            sample_questions.append(question)
            # print(question)
    # return array with count sample questions
    else:
        for i in range(count):
            sample_questions.append(questions[language][i])
            # print(questions[language][i])
    return sample_questions
# ask a question and receive a response
def ask_question(language, question=""):
    if language not in questions.keys():
        # return error message that specific language is not available
        return []
    
    if question == "":
        # generate a random question
        index = random.randint(0, len(questions[language])-1)
        question = questions[language][index]
        # print("Random question:",question)
    
    index = random.randint(0, len(answers[language])-1)
    answer = answers[language][index]
    return (question, answer)
    # shake_ball(language)
    # print(answer)

# shake the ball to get another answer to the previous question, specify language and optionally the amount of time to shake
def shake_ball(language, shake_time=10):
    if language not in questions.keys():
        # return error message that specific language is not available
        return "invalid"
    time.sleep(shake_time)
    index = random.randint(0, len(answers[language])-1)
    answer = answers[language][index]
    return answer

# similar to get_questions, return list of all sample answers or to specified count
def get_answers(language, count=0):
    if language not in questions.keys():
        # return error message that specific language is not available
        return []
    
    # handle invalid count
    if count < 0:
        count = 0

    sample_answers = []
    if count == 0 or count > len(answers[language]):
        for answer in answers[language]:
            sample_answers.append(answer)
            # print(answer)
    else:
        for i in range(count):
            sample_answers.append(answers[language][i])
            # print(answers[language][i])
    return sample_answers



import unittest, time

from src.magic_eight.magic_eight import get_answers, get_questions, ask_question, shake_ball, questions, answers

class TestMagicEight(unittest.TestCase):
## testing get_answers
    def test_get_answers_just_language(self):
        """Test the get_answers function with just language input"""
        expected = answers["en"]
        result = get_answers("en")
        self.assertEqual(result, expected)

    def test_get_answers_language_and_valid_count(self):
        """Test the get_answers function with language and count input"""
        expected = [
            "¡Sí, definitivamente!",
            "Es cierto.",
            "Sin duda.",
        ]
        result = get_answers("es", 3)
        self.assertEqual(result, expected)

    def test_get_answers_language_and_invalid_count(self):
        """Test the get_answers function with language and count input beyond size of answers[]"""
        expected = answers["fr"]
        result = get_answers("fr",-3)
        self.assertEqual(result, expected)
        
## testing get_questions
    def test_get_questions_just_language(self):
        """Test the get_questions function with just language input"""
        expected = questions["en"]
        result = get_questions("en")
        self.assertEqual(result, expected)

    def test_get_questions_language_and_valid_count(self):
        """Test the get_questions function with language and count input"""
        expected = [
            "Vais-je passer une bonne journée aujourd'hui?",
        ]
        result = get_questions("fr", 1)
        self.assertEqual(result, expected)

    def test_get_questions_language_and_invalid_count(self):
        """Test the get_questions function with language and count input beyond size of questions[]"""
        expected = questions["es"]
        result = get_questions("es",100)
        self.assertEqual(result, expected)

## testing ask_question
    def test_ask_question_specific(self):
        """Test ask_question with a specific question"""
        question, answer = ask_question("en", "Will I pass this class")
        expected_question = "Will I pass this class"
        expected = answers["en"]
        self.assertEqual(question, expected_question)
        self.assertIn(answer, expected)

    def test_ask_question_random(self):
        """Test ask_question with random question generation"""
        question, answer = ask_question("es")
        expected_questions = questions["es"]
        expected_answer = answers["es"]
        self.assertIn(question, expected_questions)
        self.assertIn(answer, expected_answer)

    def test_ask_question_invalid_language(self):
        """Test ask_question with an invalid language"""
        result = ask_question("frr") 
        self.assertEqual(result, [])

## testing shake_ball 
    def test_shake_ball_no_delay(self):
        """Test shake_ball with a valid language and no delay"""
        result = shake_ball("fr", 0)
        expected = answers["fr"]
        self.assertIn(result, expected)

    def test_shake_ball_invalid_language(self):
        """Test shake_ball with an invalid language"""
        result = shake_ball("it", 0)
        self.assertEqual(result, "invalid")

    def test_shake_ball_with_delay(self):
        """Test shake_ball with a valid language and 1 sec shake time"""
        expected = answers["es"]
        start_time = time.time()
        result = shake_ball("es", 2)
        delay_time = time.time() - start_time
        self.assertIn(result, expected)
        # make sure the delay time calculated is less not more than .1 seconds than expected to account for overhead
        self.assertLessEqual(delay_time - 2, .1)

    def test_shake_ball_non_integer_shake_time(self):
        """Test shake_ball with a non-integer shake time (i.e. a float)"""
        expected = answers["fr"]
        start_time = time.time()
        result = shake_ball("fr", 2.1) 
        delay_time = time.time() - start_time
        self.assertIn(result, expected)
        # make sure the delay time calculated is less not more than .1 seconds than expected to account for overhead
        self.assertLessEqual(delay_time - 2.1, .1)

if __name__ == "__main__":
    unittest.main()


answers = {
    "en": [
        "Yes, definitely!",
        "It is certain.",
        "Without a doubt.",
        "You may rely on it.",
        "As I see it, yes.",
        "Most likely.",
        "Outlook good.",
        "Signs point to yes.",
        "Yes.",
        "Better not tell you now.",
        "Ask again later.",
        "Cannot predict now.",
        "Don’t count on it.",
        "My sources say no.",
        "Outlook not so good.",
        "Very doubtful.",
        "Concentrate and ask again.",
        "Reply hazy, try again.",
        "It is decidedly so.",
        "All signs point to yes!"
    ],
    "fr": [
        "Oui, absolument!",
        "C'est certain.",
        "Sans aucun doute.",
        "Vous pouvez compter dessus.",
        "D'après moi, oui.",
        "Très probable.",
        "Les perspectives sont bonnes.",
        "Les signes indiquent que oui.",
        "Oui.",
        "Mieux vaut ne pas te le dire maintenant.",
        "Demande encore plus tard.",
        "Impossible de prédire maintenant.",
        "Ne compte pas là-dessus.",
        "Mes sources disent non.",
        "Les perspectives ne sont pas très bonnes.",
        "Très douteux.",
        "Concentre-toi et demande encore.",
        "Réponse floue, essaie encore.",
        "C'est décidément ainsi.",
        "Tous les signes indiquent que oui!"
    ],
    "es": [
        "¡Sí, definitivamente!",
        "Es cierto.",
        "Sin duda.",
        "Puedes confiar en ello.",
        "Según veo, sí.",
        "Muy probable.",
        "Las perspectivas son buenas.",
        "Las señales apuntan a que sí.",
        "Sí.",
        "Mejor no decirte ahora.",
        "Pregunta de nuevo más tarde.",
        "No puedo predecir ahora.",
        "No cuentes con ello.",
        "Mis fuentes dicen que no.",
        "Las perspectivas no son muy buenas.",
        "Muy dudoso.",
        "Concéntrate y pregunta de nuevo.",
        "Respuesta confusa, intenta otra vez.",
        "Es decididamente así.",
        "¡Todos los signos apuntan a que sí!"
    ]
}



questions = {
    "en": [
        "Will I have a good day today?",
        "Should I go for that job interview?",
        "Is it a good time to start a new project?",
        "Will I find true love soon?",
        "Should I ask them out?",
        "Will I ace my test tomorrow?",
        "Is my crush thinking about me?",
        "Is today my lucky day?",
        "Should I take a vacation?",
        "Will I get a promotion this year?",
        "Is it time to try a new hobby?",
        "Will I become wealthy?",
        "Should I order takeout tonight?",
        "Is it a good idea to start my own business?",
        "Will my team win the game?",
        "Is now the right time to make a big change?",
        "Will I be successful in my career?",
        "Should I watch that new movie?",
        "Will I achieve my goals this year?",
        "Should I trust that person?"
    ],
    "fr": [
        "Vais-je passer une bonne journée aujourd'hui?",
        "Devrais-je passer cet entretien d'embauche?",
        "Est-ce un bon moment pour commencer un nouveau projet?",
        "Vais-je trouver le grand amour bientôt?",
        "Devrais-je lui proposer de sortir?",
        "Vais-je réussir mon examen demain?",
        "Mon coup de cœur pense-t-il à moi?",
        "Est-ce mon jour de chance aujourd'hui?",
        "Devrais-je prendre des vacances?",
        "Vais-je obtenir une promotion cette année?",
        "Est-il temps d'essayer un nouveau passe-temps?",
        "Vais-je devenir riche?",
        "Devrais-je commander à emporter ce soir?",
        "Est-ce une bonne idée de créer ma propre entreprise?",
        "Mon équipe va-t-elle gagner le match?",
        "Est-ce le bon moment pour faire un grand changement?",
        "Vais-je réussir dans ma carrière?",
        "Devrais-je regarder ce nouveau film?",
        "Vais-je atteindre mes objectifs cette année?",
        "Devrais-je faire confiance à cette personne?"
    ],
    "es": [
        "¿Tendré un buen día hoy?",
        "¿Debería ir a esa entrevista de trabajo?",
        "¿Es un buen momento para comenzar un nuevo proyecto?",
        "¿Encontraré el verdadero amor pronto?",
        "¿Debería invitarles a salir?",
        "¿Aprobaré mi examen mañana?",
        "¿Está mi crush pensando en mí?",
        "¿Es hoy mi día de suerte?",
        "¿Debería tomar unas vacaciones?",
        "¿Recibiré una promoción este año?",
        "¿Es momento de probar un nuevo pasatiempo?",
        "¿Me volveré rico?",
        "¿Debería pedir comida para llevar esta noche?",
        "¿Es una buena idea iniciar mi propio negocio?",
        "¿Ganará mi equipo el partido?",
        "¿Es ahora el momento adecuado para hacer un gran cambio?",
        "¿Tendré éxito en mi carrera?",
        "¿Debería ver esa nueva película?",
        "¿Lograré mis objetivos este año?",
        "¿Debería confiar en esa persona?"
    ]
}